package lightcontainer.storage;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * AppData represents the database of the FileFrontEnd program.
 * It contains an AppConfig instance and a collection of Users.
 *
 * @author Maximilien LEDOUX <m.ledoux@student.helmo.be>
 * @version 1.0
 * @see AppConfig
 * @see User
 * @since 1.0
 */
public class AppData {

    private static AppData instance = null;
    private AppConfig appConfig;
    private final Map<String, User> users;


    /**
     * Constructs a new instance of AppData.
     * Sets appConfig to null and creates a new Hashmap of users.
     */
    private AppData() {
        this.appConfig = null;
        this.users = new HashMap<>();
    }

    /**
     * @return An instance of this class. Always returns the same instance.
     */
    public static AppData getInstance() {
        if (instance == null) {
            instance = new AppData();
        }
        return instance;
    }

    /**
     * @return The AppConfig
     */
    public AppConfig getAppConfig() {
        return appConfig;
    }

    /**
     * Sets the AppConfig. This method sets the AppConfig for once and for all.
     * It is locked after first call.
     *
     * @param appConfig The network configuration of the program.
     */
    public void setAppConfig(AppConfig appConfig) {
        if (this.appConfig == null) {
            this.appConfig = appConfig;
        }
    }

    /**
     * @param userName The name of the user.
     * @return The user corresponding to userName, null otherwise.
     */
    public User getUser(String userName) {
        return this.users.get(userName);
    }




    /**
     * Use this method when a user signs up.
     *
     * @return True if the user was added. False if a user with the same name already exists.
     */
    public boolean addUser(String login, String password, String key) {
        User user = new User(login, password, key, new HashMap<>());
        if (this.users.containsKey(user.getName())) {
            return false;
        } else {
            this.users.put(user.getName(), user);
            return true;
        }
    }

    public Iterator<User> usersIterator() {
        return users.values().iterator();
    }

    /**
     * @param fileName The name of the file
     * @param user     The user
     * @return The file corresponding to the given name and belonging to the user. Null if the user cannot be found or the file cannot be found
     * @deprecated Maybe not useful. DO NOT USE FOR THE TIME BEING
     */
    public File getFileOf(String fileName, User user) {
        return this.users.get(user.getName()).getFile(fileName);
    }

    /**
     * Call this method after receiving SAVEFILE_OK from the StorBackEnd.
     * Do NOT call when receiving SAVEFILE_ERROR, or it will break the system's synchronization.
     * <p>
     * Adds the file of for a specific user.
     * True indicates the success of the operation.
     * False indicates the failure of the operation.
     *
     * @param file The file to add
     * @param user The user who wants to add the file
     * @return True if the user is found and a file with the same name doesn't already exist for this user. False otherwise.
     */
    public boolean addFileFor(File file, User user) {
        if (!this.users.containsKey(user.getName())) {
            return false;
        } else {
            this.users.get(user.getName()).addFile(file);
            return true;
        }
    }

    /**
     * Call this method after receiving REMOVEFILE_OK from the StorBackEnd.
     * Do NOT call when receiving REMOVEFILE_ERROR, or it will break the system's synchronization.
     * Deletes the file of for a specific user.
     * True indicates the success of the operation.
     * False indicates the failure of the operation.
     *
     * @param fileName The name of the file to delete
     * @param user     The user who wants to delete the file
     * @return True if the user is found and the file was deleted. False otherwise.
     */
    public boolean deleteFileOf(String fileName, User user) {
        if (!this.users.containsKey(user.getName())) {
            return false;
        } else {
            return this.users.get(user.getName()).deleteFile(fileName);
        }
    }

    /**
     * @param user    The user who wants to add a storage for their file
     * @param file    The file that needs a new storage
     * @param storage The storage to add
     * @return True if the storage was added. False otherwise.
     */
    public boolean addStorage(User user, File file, String storage) {
        if (!this.users.containsKey(user.getName())) {
            return false;
        } else {
            return this.users.get(user.getName()).addStorage(file, storage);
        }
    }

}
